/**************************************************************************//**
*   @file   main.c
*   @brief  System validation program
*   @author ACostina (adrian.costina@analog.com)
*
*******************************************************************************
* Copyright 2012(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 1528
******************************************************************************/
/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <altera_avalon_pio_regs.h>
#include "system.h"
#include "adi.h"
#include "AD7327.h"
#define ADC_StartAcquisition AD7327_StartAcquisition
#define ADC_ReadStatus AD7327_ReadStatus
/*****************************************************************************/
/************************ Constants Definitions ******************************/
/*****************************************************************************/
#define SAMPLE_NO 8*1024
/*****************************************************************************/
/************************ Variables Definitions ******************************/
/*****************************************************************************/
static unsigned int txData[SAMPLE_NO];              //array to store the data read from the ADC
static unsigned int size            = 0;            //used for the request of number of samples
volatile char  startAcquisition     = 0;            //starts one acquisition cycle
/*****************************************************************************/
/************************ Functions Definitions ******************************/
/*****************************************************************************/
int main(void)
{
    volatile char* startAcquisitionTCL           = (char*)0x0001AFE0;   // signals that acquisition has been started
    volatile char* transferCompleteTCL           = (char*)0x0001AFE1;   // signals that transfer of data between PC and BeMicro is completed
    volatile char* processingDataTCL             = (char*)0x0001AFE2;   // signals that the TCL has finished the conversion from binary to decimal values
    volatile char* operationCompleteTCL          = (char*)0x0001AFE3;   // signals that this iteration of acquisition is complete
    volatile int* command                        = (int*) 0x0001AFE4;   // used for controlling the driver through HDL
    volatile int*  dataArrayAddr                 = (int*) 0x0001AFE8;   // pointer to the memory zone where the acquisitions will be stored
    volatile int*  startAcqPointer               = (int*) 0x0001AFEC;   // pointer used to start the acquisition process from the TCL

    volatile int old_command;                                           // variable used for monitoring the configuration word from the TCL script

    *dataArrayAddr  = txData;           // read the address of the start of the memory allocated for ADC data
    *command        = 0xA800;           // default configuration of the ADC
    old_command     = *command;         // store the command sent to the ADC peripheral

    AD7327_Reset();
    AD7327_WriteRegister( *command);    // configure the HDL driver
    usleep(100);                        // wait for the configuration to be performed

    //signal the start of the program
    *startAcqPointer                    = &startAcquisition;

    while(1)
    {
        *startAcquisitionTCL            = 0;
        *transferCompleteTCL            = 0;
        *processingDataTCL              = 0;
        *operationCompleteTCL           = 0;

        ToggleLed(LED_1);
        size = SAMPLE_NO;
        if (*command != old_command)
        {
            old_command = *command;             // store the command sent to the ADC peripheral
            AD7327_WriteRegister( *command);    // configure the HDL driver
            usleep(10);                     // wait for the configuration to be performed
        }

        if(startAcquisition)
        {

            startAcquisition     = 0;
            //signal the start of data acquisition
            SetLed(LED_1);
            ADC_StartAcquisition(size, txData);

            while(!(ADC_ReadStatus() & 0x00000001));
            //signal the start of data transfer to the PC
            SetLed(LED_2);
            *startAcquisitionTCL = 1;

            //signal the end of data transfer to the PC
            while (*transferCompleteTCL == 0);
            SetLed(LED_3);

            //signal the end of data conversion to decimal values
            while (*processingDataTCL == 0);
            SetLed(LED_4);

            //signal the end of data transfer to a file
            while (*operationCompleteTCL == 0);
            ClearLed(LED_ALL);
        }
    }
}
